@@ -15,6 +15,8 @@
 #   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #   See the License for the specific language governing permissions and
 #   limitations under the License.
+from __future__ import absolute_import
+
 import re
 import base64
 import itertools
@@ -45,6 +47,33 @@ class SYSTEM_TOKENS:
     StartOfStreamToken = 0x0F
 
 
+class NODE_TYPES:
+    NULL = 0x00
+    WSTRING = 0x01
+    STRING = 0x02
+    SIGNED_BYTE = 0x03
+    UNSIGNED_BYTE = 0x04
+    SIGNED_WORD = 0x05
+    UNSIGNED_WORD = 0x06
+    SIGNED_DWORD = 0x07
+    UNSIGNED_DWORD = 0x08
+    SIGNED_QWORD = 0x09
+    UNSIGNED_QWORD = 0x0A
+    FLOAT = 0x0B
+    DOUBLE = 0x0C
+    BOOLEAN = 0x0D
+    BINARY = 0x0E
+    GUID = 0x0F
+    SIZE = 0x10
+    FILETIME = 0x11
+    SYSTEMTIME = 0x12
+    SID = 0x13
+    HEX32 = 0x14
+    HEX64 = 0x15
+    BXML = 0x21
+    WSTRINGARRAY = 0x81
+
+
 node_dispatch_table = []  # updated at end of file
 node_readable_tokens = []  # updated at end of file
 
@@ -410,30 +439,30 @@ def get_variant_value(buf, offset, chunk, parent, type_, length=None):
       buffer and offset.
     """
     types = {
-        0x00: NullTypeNode,
-        0x01: WstringTypeNode,
-        0x02: StringTypeNode,
-        0x03: SignedByteTypeNode,
-        0x04: UnsignedByteTypeNode,
-        0x05: SignedWordTypeNode,
-        0x06: UnsignedWordTypeNode,
-        0x07: SignedDwordTypeNode,
-        0x08: UnsignedDwordTypeNode,
-        0x09: SignedQwordTypeNode,
-        0x0A: UnsignedQwordTypeNode,
-        0x0B: FloatTypeNode,
-        0x0C: DoubleTypeNode,
-        0x0D: BooleanTypeNode,
-        0x0E: BinaryTypeNode,
-        0x0F: GuidTypeNode,
-        0x10: SizeTypeNode,
-        0x11: FiletimeTypeNode,
-        0x12: SystemtimeTypeNode,
-        0x13: SIDTypeNode,
-        0x14: Hex32TypeNode,
-        0x15: Hex64TypeNode,
-        0x21: BXmlTypeNode,
-        0x81: WstringArrayTypeNode,
+        NODE_TYPES.NULL: NullTypeNode,
+        NODE_TYPES.WSTRING: WstringTypeNode,
+        NODE_TYPES.STRING: StringTypeNode,
+        NODE_TYPES.SIGNED_BYTE: SignedByteTypeNode,
+        NODE_TYPES.UNSIGNED_BYTE: UnsignedByteTypeNode,
+        NODE_TYPES.SIGNED_WORD: SignedWordTypeNode,
+        NODE_TYPES.UNSIGNED_WORD: UnsignedWordTypeNode,
+        NODE_TYPES.SIGNED_DWORD: SignedDwordTypeNode,
+        NODE_TYPES.UNSIGNED_DWORD: UnsignedDwordTypeNode,
+        NODE_TYPES.SIGNED_QWORD: SignedQwordTypeNode,
+        NODE_TYPES.UNSIGNED_QWORD: UnsignedQwordTypeNode,
+        NODE_TYPES.FLOAT: FloatTypeNode,
+        NODE_TYPES.DOUBLE: DoubleTypeNode,
+        NODE_TYPES.BOOLEAN: BooleanTypeNode,
+        NODE_TYPES.BINARY: BinaryTypeNode,
+        NODE_TYPES.GUID: GuidTypeNode,
+        NODE_TYPES.SIZE: SizeTypeNode,
+        NODE_TYPES.FILETIME: FiletimeTypeNode,
+        NODE_TYPES.SYSTEMTIME: SystemtimeTypeNode,
+        NODE_TYPES.SID: SIDTypeNode,
+        NODE_TYPES.HEX32: Hex32TypeNode,
+        NODE_TYPES.HEX64: Hex64TypeNode,
+        NODE_TYPES.BXML: BXmlTypeNode,
+        NODE_TYPES.WSTRINGARRAY: WstringArrayTypeNode,
     }
     try:
         TypeClass = types[type_]
@@ -573,6 +602,40 @@ class CDataSectionNode(BXmlNode):
             self.token() & 0x0F == SYSTEM_TOKENS.CDataSectionToken
 
 
+class CharacterReferenceNode(BXmlNode):
+    """
+    The binary XML node for the system token 0x08.
+
+    This is an character reference node.  That is, something that represents
+      a non-XML character, eg. & --> &#x0038;.
+    """
+    def __init__(self, buf, offset, chunk, parent):
+        super(CharacterReferenceNode, self).__init__(buf, offset, chunk, parent)
+        self.declare_field("byte", "token", 0x0)
+        self.declare_field("word", "entity")
+        self._tag_length = 3
+
+    def __repr__(self):
+        return "CharacterReferenceNode(buf={!r}, offset={!r}, chunk={!r}, parent={!r})".format(
+            self._buf, self.offset(), self._chunk, self._parent)
+
+    def __str__(self):
+        return "CharacterReferenceNode(offset={}, length={}, token={})".format(
+            hex(self.offset()), hex(self.length()), hex(0x08))
+
+    def entity_reference(self):
+        return '&#x%04x;' % (self.entity())
+
+    def flags(self):
+        return self.token() >> 4
+
+    def tag_length(self):
+        return self._tag_length
+
+    def children(self):
+        return []
+
+
 class EntityReferenceNode(BXmlNode):
     """
     The binary XML node for the system token 0x09.
@@ -593,7 +656,6 @@ class EntityReferenceNode(BXmlNode):
                                                 parent=self)
             self._tag_length += new_string.length()
 
-
     def __repr__(self):
         return "EntityReferenceNode(buf={!r}, offset={!r}, chunk={!r}, parent={!r})".format(
             self._buf, self.offset(), self._chunk, self._parent)
@@ -902,159 +964,31 @@ class RootNode(BXmlNode):
 
         return self.tag_length() + children_length
 
-    def fast_template_instance(self):
+    def template_instance(self):
+        '''
+        parse the template instance node.
+        this is used to compute the location of the template definition structure.
+
+        Returns:
+          TemplateInstanceNode: the template instance.
+        '''
         ofs = self.offset()
         if self.unpack_byte(0x0) & 0x0F == 0xF:
             ofs += 4
         return TemplateInstanceNode(self._buf, ofs, self._chunk, self)
 
-    @memoize
-    def fast_substitutions(self):
-        """
-        Get the list of elements that are the
-          the substitutions for this root node.
-          Each element is one of:
-            str
-            int
-            float
-            RootNode
-        @rtype: list
-        """
-        sub_decl = []
-        sub_def = []
-        ofs = self.tag_and_children_length()
-        sub_count = self.unpack_dword(ofs)
-        ofs += 4
-        for _ in range(sub_count):
-            size = self.unpack_word(ofs)
-            type_ = self.unpack_byte(ofs + 0x2)
-            sub_decl.append((size, type_))
-            ofs += 4
-        for (size, type_) in sub_decl:
-            #[0] = parse_null_type_node,
-            if type_ == 0x0:
-                value = None
-                sub_def.append(value)
-            #[1] = parse_wstring_type_node,
-            elif type_ == 0x1:
-                s = self.unpack_wstring(ofs, size // 2).rstrip("\x00")
-                value = s.replace("<", "&gt;").replace(">", "&lt;")
-                sub_def.append(value)
-            #[2] = parse_string_type_node,
-            elif type_ == 0x2:
-                s = self.unpack_string(ofs, size)
-                value = s.decode("utf8").rstrip("\x00")
-                value = value.replace("<", "&gt;")
-                value = value.replace(">", "&lt;")
-                sub_def.append(value)
-            #[3] = parse_signed_byte_type_node,
-            elif type_ == 0x3:
-                sub_def.append(self.unpack_int8(ofs))
-            #[4] = parse_unsigned_byte_type_node,
-            elif type_ == 0x4:
-                sub_def.append(self.unpack_byte(ofs))
-            #[5] = parse_signed_word_type_node,
-            elif type_ == 0x5:
-                sub_def.append(self.unpack_int16(ofs))
-            #[6] = parse_unsigned_word_type_node,
-            elif type_ == 0x6:
-                sub_def.append(self.unpack_word(ofs))
-            #[7] = parse_signed_dword_type_node,
-            elif type_ == 0x7:
-                sub_def.append(self.unpack_int32(ofs))
-            #[8] = parse_unsigned_dword_type_node,
-            elif type_ == 0x8:
-                sub_def.append(self.unpack_dword(ofs))
-            #[9] = parse_signed_qword_type_node,
-            elif type_ == 0x9:
-                sub_def.append(self.unpack_int64(ofs))
-            #[10] = parse_unsigned_qword_type_node,
-            elif type_ == 0xA:
-                sub_def.append(self.unpack_qword(ofs))
-            #[11] = parse_float_type_node,
-            elif type_ == 0xB:
-                sub_def.append(self.unpack_float(ofs))
-            #[12] = parse_double_type_node,
-            elif type_ == 0xC:
-                sub_def.append(self.unpack_double(ofs))
-            #[13] = parse_boolean_type_node,
-            elif type_ == 0xD:
-                sub_def.append(str(self.unpack_word(ofs) > 1))
-            #[14] = parse_binary_type_node,
-            elif type_ == 0xE:
-                sub_def.append(base64.b64encode(self.unpack_binary(ofs, size)))
-            #[15] = parse_guid_type_node,
-            elif type_ == 0xF:
-                sub_def.append('{' + self.unpack_guid(ofs) + '}')
-            #[16] = parse_size_type_node,
-            elif type_ == 0x10:
-                if size == 0x4:
-                    sub_def.append(self.unpack_dword(ofs))
-                elif size == 0x8:
-                    sub_def.append(self.unpack_qword(ofs))
-                else:
-                    raise UnexpectedStateException("Unexpected size for SizeTypeNode: {}".format(hex(size)))
-            #[17] = parse_filetime_type_node,
-            elif type_ == 0x11:
-                sub_def.append(self.unpack_filetime(ofs))
-            #[18] = parse_systemtime_type_node,
-            elif type_ == 0x12:
-                sub_def.append(self.unpack_systemtime(ofs))
-            #[19] = parse_sid_type_node,  -- SIDTypeNode, 0x13
-            elif type_ == 0x13:
-                version = self.unpack_byte(ofs)
-                num_elements = self.unpack_byte(ofs + 1)
-                id_high = self.unpack_dword_be(ofs + 2)
-                id_low = self.unpack_word_be(ofs + 6)
-                value = "S-{}-{}".format(version, (id_high << 16) ^ id_low)
-                for i in range(num_elements):
-                    val = self.unpack_dword(ofs + 8 + (4 * i))
-                    value += "-{}".format(val)
-                sub_def.append(value)
-            #[20] = parse_hex32_type_node,  -- Hex32TypeNode, 0x14
-            elif type_ == 0x14:
-                value = "0x"
-                b = self.unpack_binary(ofs, size)[::-1]
-                for i in range(len(b) - 1):
-                    value += '{:02x}'.format(six.indexbytes(b, i))
-                sub_def.append(value)
-            #[21] = parse_hex64_type_node,  -- Hex64TypeNode, 0x15
-            elif type_ == 0x15:
-                value = "0x"
-                b = self.unpack_binary(ofs, size)[::-1]
-                for i in range(len(b) - 1):
-                    value += '{:02x}'.format(six.indexbytes(b, i))
-                sub_def.append(value)
-            #[33] = parse_bxml_type_node,  -- BXmlTypeNode, 0x21
-            elif type_ == 0x21:
-                sub_def.append(RootNode(self._buf, self.offset() + ofs,
-                                        self._chunk, self))
-            #[129] = TODO, -- WstringArrayTypeNode, 0x81
-            elif type_ == 0x81:
-                bin = self.unpack_binary(ofs, size)
-                acc = []
-                while len(bin) > 0:
-                    match = re.search(b"((?:[^\x00].)+)", bin)
-                    if match:
-                        frag = match.group()
-                        acc.append("<string>")
-                        acc.append(frag.decode("utf16"))
-                        acc.append("</string>\n")
-                        bin = bin[len(frag) + 2:]
-                        if len(bin) == 0:
-                            break
-                    frag = re.search(b"(\x00*)", bin).group()
-                    if len(frag) % 2 == 0:
-                        for _ in range(len(frag) // 2):
-                            acc.append("<string></string>\n")
-                    else:
-                        raise ParseException("Error parsing uneven substring of NULLs")
-                    bin = bin[len(frag):]
-                sub_def.append("".join(acc))
-            else:
-                raise "Unexpected type encountered: {}".format(hex(type_))
-            ofs += size
-        return sub_def
+    def template(self):
+        '''
+        parse the template referenced by this root node.
+        note, this template structure is not guaranteed to be located within the root node's boundaries.
+
+        Returns:
+          TemplateNode: the template.
+        '''
+        instance = self.template_instance()
+        offset = self._chunk.offset() + instance.template_offset()
+        node = TemplateNode(self._buf, offset, self._chunk, instance)
+        return node
 
     @memoize
     def substitutions(self):
@@ -1131,7 +1065,8 @@ class VariantTypeNode(BXmlNode):
         raise NotImplementedError("string not implemented for {!r}".format(self))
 
 
-class NullTypeNode(object):  # but satisfies the contract of VariantTypeNode, BXmlNode, but not Block
+# but satisfies the contract of VariantTypeNode, BXmlNode, but not Block
+class NullTypeNode(object):
     """
     Variant type 0x00.
     """
@@ -1294,7 +1229,7 @@ class UnsignedDwordTypeNode(VariantTypeNode):
     """
     def __init__(self, buf, offset, chunk, parent, length=None):
         super(UnsignedDwordTypeNode, self).__init__(buf, offset,
-                                                   chunk, parent,
+                                                    chunk, parent,
                                                     length=length)
         self.declare_field("dword", "dword", 0x0)
 
@@ -1327,7 +1262,7 @@ class UnsignedQwordTypeNode(VariantTypeNode):
     """
     def __init__(self, buf, offset, chunk, parent, length=None):
         super(UnsignedQwordTypeNode, self).__init__(buf, offset,
-                                                   chunk, parent,
+                                                    chunk, parent,
                                                     length=length)
         self.declare_field("qword", "qword", 0x0)
 
@@ -1383,7 +1318,7 @@ class BooleanTypeNode(VariantTypeNode):
         return 4
 
     def string(self):
-        if self.int32 > 0:
+        if self.int32() > 0:
             return "True"
         return "False"
 
@@ -1409,7 +1344,7 @@ class BinaryTypeNode(VariantTypeNode):
         return self._length
 
     def string(self):
-        return base64.b64encode(self.binary())
+        return base64.b64encode(self.binary()).decode('ascii')
 
 
 class GuidTypeNode(VariantTypeNode):
@@ -1425,7 +1360,6 @@ class GuidTypeNode(VariantTypeNode):
         return 16
 
     def string(self):
-        print('{' + self.guid() + '}')
         return '{' + self.guid() + '}'
 
 
@@ -1464,7 +1398,7 @@ class FiletimeTypeNode(VariantTypeNode):
         self.declare_field("filetime", "filetime", 0x0)
 
     def string(self):
-        return self.filetime().isoformat("T") + "Z"
+        return self.filetime().isoformat(' ')
 
     def tag_length(self):
         return 8
@@ -1483,7 +1417,7 @@ class SystemtimeTypeNode(VariantTypeNode):
         return 16
 
     def string(self):
-        return self.systemtime().isoformat("T") + "Z"
+        return self.systemtime().isoformat(' ')
 
 
 class SIDTypeNode(VariantTypeNode):
@@ -1534,8 +1468,9 @@ class Hex32TypeNode(VariantTypeNode):
 
     def string(self):
         ret = "0x"
-        for c in self.hex()[::-1]:
-            ret += "{:02x}".format(ord(c))
+        b = self.hex()[::-1]
+        for i in range(len(b)):
+            ret += '{:02x}'.format(six.indexbytes(b, i))
         return ret
 
 
@@ -1553,8 +1488,9 @@ class Hex64TypeNode(VariantTypeNode):
 
     def string(self):
         ret = "0x"
-        for c in self.hex()[::-1]:
-            ret += "{:02x}".format(c)
+        b = self.hex()[::-1]
+        for i in range(len(b)):
+            ret += '{:02x}'.format(six.indexbytes(b, i))
         return ret
 
 
@@ -1571,7 +1507,7 @@ class BXmlTypeNode(VariantTypeNode):
         return self._length or self._root.length()
 
     def string(self):
-        return str(self._root)
+        return ''
 
     def root(self):
         return self._root
@@ -1583,7 +1519,7 @@ class WstringArrayTypeNode(VariantTypeNode):
     """
     def __init__(self, buf, offset, chunk, parent, length=None):
         super(WstringArrayTypeNode, self).__init__(buf, offset, chunk,
-                                              parent, length=length)
+                                                   parent, length=length)
         if self._length is None:
             self.declare_field("word",   "binary_length", 0x0)
             self.declare_field("binary", "binary",
@@ -1598,25 +1534,25 @@ class WstringArrayTypeNode(VariantTypeNode):
         return self._length
 
     def string(self):
-        bin = self.binary()
+        binary = self.binary()
         acc = []
-        while len(bin) > 0:
-            match = re.search("((?:[^\x00].)+)", bin)
+        while len(binary) > 0:
+            match = re.search(b"((?:[^\x00].)+)", binary)
             if match:
                 frag = match.group()
                 acc.append("<string>")
                 acc.append(frag.decode("utf16"))
                 acc.append("</string>\n")
-                bin = bin[len(frag) + 2:]
-                if len(bin) == 0:
+                binary = binary[len(frag) + 2:]
+                if len(binary) == 0:
                     break
-            frag = re.search("(\x00*)", bin).group()
+            frag = re.search(b"(\x00*)", binary).group()
             if len(frag) % 2 == 0:
                 for _ in range(len(frag) // 2):
                     acc.append("<string></string>\n")
             else:
                 raise ParseException("Error parsing uneven substring of NULLs")
-            bin = bin[len(frag):]
+            binary = binary[len(frag):]
         return "".join(acc)
 
 
@@ -1629,7 +1565,7 @@ node_dispatch_table = [
     ValueNode,
     AttributeNode,
     CDataSectionNode,
-    None,
+    CharacterReferenceNode,
     EntityReferenceNode,
     ProcessingInstructionTargetNode,
     ProcessingInstructionDataNode,
